--[[
--Last Man On the Hill v1.0
--By Darren Watts (W'rkncacnter)
--]]

spawns = {}
timer = {}
score = {}
hill = {}
waiting = {}
just_reset = true
reset_timer = 40
positions = {}
TIMER_INIT_AMT = 600
TIMER_AMOUNT = 450
last_player_to_die = -1
about_to_reset_timer = 0

function init()
	for i = 1, number_of_players(), 1 do
		spawns[i] = get_player_polygon(i-1)
		timer[i] = TIMER_INIT_AMT
		score[i] = 0
		positions[i] = {}
		waiting[i] = false
	end
	for i = 0, number_of_polygons() - 1, 1 do
		if(get_polygon_type(i) == _polygon_is_hill) then
			hill[table.getn(hill) + 1] = i
		end
	end
end

function idle()
	set_score()
	halt_player()
	update_reset_timer()
	update_timer()
	kill_player()
	use_overlay()
	player_stays_dead()
end

function set_score()
	for i = 0, number_of_players() - 1, 1 do
		set_points(i, score[i+1])
	end
end

function do_nothing() end

function is_hill(poly)
	for i = 1, table.getn(hill), 1 do
		if(hill[i] == poly) then
			return true
		end
	end
	return false
end

function reset(player)
	just_reset = true
	reset_timer = 40
	screen_print(get_player_name (player).." wins the round!")
	score[player+1] = score[player+1] + 30
	for i = 0, number_of_players() - 1, 1 do
		timer[i+1] = TIMER_INIT_AMT
		waiting[i+1] = false
		if(get_life(i) < 150) then
			set_life(i, 150)
		end
	end
	local poly = math.mod (better_random (), table.getn(spawns))
	teleport_player (player, spawns[poly+1])
end

function halt_player()
	for i = 0, number_of_players() - 1, 1 do
		if(waiting[i+1] and not player_is_dead(i)) then
			select_weapon(i, _item_fist)
			set_player_position(i, positions[i+1][1], positions[i+1][2], 
				positions[i+1][3], positions[i+1][4])
		end
	end
end

function player_killed(player, aggressor_player, action, projectile)
	local was_waiting = false
	if(waiting[player+1] == true) then
		was_waiting = true
	end
	waiting[player+1] = true
	about_to_be_reset_timer = 5
	if(player ~= aggressor_player and not only_one_not_waiting(player)) then 
		last_player_to_die = player
		if(not was_waiting and not only_one_not_waiting(aggressor_player)) then
			screen_print(get_player_name(player).." has been eliminated from the round!")
		end
	elseif(player ~= aggressor_player and only_one_not_waiting(player)
		and not just_reset) then
		last_player_to_die = player
		reset(player)
	elseif(player == aggressor_player and only_one_not_waiting(player)
		and not just_reset) then 
		reset(last_player_to_die)
	end
end

function player_revived(player)
	if(waiting[player+1]) then
		inflict_damage(player, 1000)
		award_kills (-1, player, -1)
		screen_print(player, "You must wait until this round is over!")
	end
	positions[player+1][1],
	positions[player+1][2],
	positions[player+1][3] = get_player_position(player)
	positions[player+1][4] = get_player_polygon(player)
end

function update_reset_timer()
	if(reset_timer > 0) then
		reset_timer = reset_timer - 1
	else
		just_reset = false
	end
	if(about_to_reset_timer > 0) then
		about_to_reset_timer = about_to_reset_timer - 1
	end
end

function update_timer()
	for i = 0, number_of_players() - 1, 1 do
		if(is_hill(get_player_polygon(i))) then
			timer[i+1] = TIMER_AMOUNT
		elseif(waiting[i+1]) then
			timer[i+1] = TIMER_INIT_AMT
		elseif(timer[i+1] > 0) then
			timer[i+1] = timer[i+1] - 1
		end
	end
end

function kill_player()
	for i = 0, number_of_players() - 1, 1 do
		if(timer[i+1] == 0 and not only_one_not_waiting(i)) then
			inflict_damage(i, 1000)
		elseif(about_to_reset_timer == 0 and only_one_not_waiting(i) and not just_reset) then
			reset(i)
		end
	end
end

function only_one_not_waiting(player)
	for i = 0, number_of_players() - 1, 1 do
		if(player == i) then do_nothing()
		elseif(waiting[i+1] == false) then return false
		end
	end
	return true
end

function player_stays_dead()
	for i = 0, number_of_players() - 1, 1 do
		if(player_is_dead(i) and timer[i+1] == 0) then
			waiting[i+1] = true
		end
	end
end

function use_overlay()
	local players_left = 0
	for i = 0, number_of_players() - 1, 1 do
		if(not waiting[i+1]) then players_left = players_left + 1 end
	end
	set_overlay_color(1, 0)
	set_overlay_text(1, "Players remaining: "..players_left)
	local time_amount = math.floor(timer[local_player_index() + 1] / 30)
	if(time_amount > 10) then
		set_overlay_color(0, 0)
	elseif(time_amount > 5) then
		set_overlay_color(0, 5)
	else set_overlay_color(0, 2)
	end
	set_overlay_text(0, time_amount)
end